/*
 * HF synthesizer
 * base on Si5351x2
 * 9 - bands
 *
 * Lcd_1602.c
 *
 * Author: R9OFG (ex R0AEK)
 * Version - 2.0 beta
 *
 * radio0aek@gmail.com
 * https://r0aek.me
 */ 

#include "Lcd_1602.h"

/*  S-*/
uint8_t S1_Char[8]	= {0b00000,0b00000,0b10101,0b01110,0b00100,0b00100,0b00100,0b00100};
uint8_t S3_Char[8]	= {0b00000,0b00000,0b00000,0b00000,0b00000,0b00000,0b11111,0b11111};
uint8_t S5_Char[8]	= {0b00000,0b00000,0b00000,0b00000,0b00000,0b11111,0b10001,0b11111};
uint8_t S7_Char[8]	= {0b00000,0b00000,0b00000,0b00000,0b11111,0b10001,0b10001,0b11111};
uint8_t S8_Char[8]	= {0b00000,0b00000,0b00000,0b11111,0b10001,0b10001,0b10001,0b11111};
uint8_t S9_Char[8]	= {0b00000,0b00000,0b11111,0b10001,0b10001,0b10001,0b10001,0b11111};
uint8_t S10_Char[8]	= {0b00000,0b11111,0b11111,0b11111,0b11111,0b11111,0b11111,0b11111};	

	
void LCD_send_half_byte (unsigned char c)
{
	E_1;						//  
	_delay_us(40);
	PORTB |= (c << 1);			//      ,     PB1-PB4
	E_0;						//  
	_delay_us(40);
	PORTB &= 0b00000000;		//   B
}

void LCD_send_byte (unsigned char c, unsigned char mode)
{
	unsigned char hc = 0;
	if (mode == 0)				//    
		RS_0;
	else
		RS_1;
	hc = (c >> 4);				//   
	LCD_send_half_byte(hc);		//   LCD   
	LCD_send_half_byte(c);		//   LCD   
}

void LCD_define_char(uint8_t *pc, uint8_t char_code)
{
	uint8_t i,a;
	a=(char_code<<3)|0x40;
	for (i=0;i<8;i++)
	{
		 LCD_send_byte(a++,0);
		 LCD_send_byte(*pc++,1);
	}
}

void LCD_send_char (unsigned char c)
{
	LCD_send_byte(c,1);
}

void LCD_send_str (char str[])
{
	wchar_t n;
	for(n=0;str[n]!='\0';n++)
		LCD_send_char(str[n]);
}

void LCD_send_str_delay (char str[])
{
	wchar_t n;
	for(n=0;str[n]!='\0';n++)
	{
		LCD_send_char(str[n]);
		_delay_ms(25); //25
	}
}

void LCD_set_pos (unsigned char x, unsigned y)
{
	char adress;
	adress=(0x40*y+x)|0b10000000;
	LCD_send_byte(adress,0);
}

void LCD_ini (void)
{
	LCD_send_half_byte(0x03);				//8   1 
	_delay_ms(40);	
	LCD_send_half_byte(0x02);				//4   1 
	_delay_ms(40);
	LCD_send_byte(0x28,0);					//4   2 
	_delay_ms(40);
	LCD_send_byte(0x06,0);					//    
	_delay_ms(40);
	
	/*  LCD    S-*/
	LCD_define_char(S1_Char,0x00);
	_delay_ms(40);
	LCD_define_char(S3_Char,0x01);
	_delay_ms(40);
	LCD_define_char(S5_Char,0x02);
	_delay_ms(40);
	LCD_define_char(S7_Char,0x03);
	_delay_ms(40);
	LCD_define_char(S8_Char,0x04);
	_delay_ms(40);
	LCD_define_char(S9_Char,0x05);
	_delay_ms(40);
	LCD_define_char(S10_Char,0x06);
	_delay_ms(40);
	/*   LCD*/
	
 	LCD_send_byte(0x0C,0);					// LCD,  
 	_delay_ms(40);
	LCD_send_byte(0x01,0);					// 
	
	//
	_delay_ms(10);
	LCD_set_pos(1,0);						//    
	_delay_ms(10);
	//LCD_send_str_delay("HF transceiver");
	LCD_send_str_delay("HF synthesizer");
	_delay_ms(750);
	LCD_set_pos(0,1);						//    
	_delay_ms(10);
	//LCD_send_str_delay("STEP-II  SMD");
	LCD_send_str_delay("   Si5351 x 2   ");
	_delay_ms(750);							//
	LCD_send_byte(0x01,0);					// 
	_delay_ms(10);
	LCD_set_pos(4,0);						//    
	_delay_ms(10);
	LCD_send_str_delay("By R9OFG");
	_delay_ms(750);
	LCD_set_pos(2,1);						//    
	LCD_send_str_delay("Ver: 2.0 (b)");
	_delay_ms(750);							//
	LCD_send_byte(0x01,0);					// 
	// */
}