/*
 * HF synthesizer
 * base on Si5351x2
 * 9 - bands
 *
 * Main.h
 *
 * Author: R9OFG (ex R0AEK)
 * Version - 2.0 beta
 *
 * radio0aek@gmail.com
 * https://r0aek.me
 */ 

#ifndef MAIN_H_
#define MAIN_H_

#define F_CPU 25000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>

#include "Lcd_1602.h"
#include "si5351a.h"
#include "i2c.h"
#include "Usart.h"

#define FREQ_VFO_MIN			1000000					//   VFO  MHz
#define FREQ_VFO_MAX			29999999				//   VFO  MHz
#define FREQ_LSB_DEF			8865000					//    LSB  
#define FREQ_USB_DEF			8868000					//    USB  
#define FREQ_CWL_DEF			8865000					//    CWL  
#define FREQ_CWU_DEF			8868000					//    CWU  
#define FREQ_XTALL_1_DEF		25000000				//  XTALL_1 Si5351  VFO  
#define FREQ_XTALL_2_DEF		25000000				//  XTALL_2 Si5351  BFO  

#define ENCODER_MASK			0x0C					//      PD2  PD3 (0b00001100)

#define SR_CLOCK				PD4						//   -  
#define SR_DATA					PD5						//   -  
#define SR_LATCH				PD6						//   - 

#define SW_I2C					PC3						//   SCL1

#define FUNC_MODE_NORMAL		1						//  -  (RX)
#define FUNC_MODE_TX			2						//  -  (TX)
#define FUNC_MODE_SET_STEP		3						//  -   
#define FUNC_MODE_SET_BFO_LSB	4						//  -   BFO_LSB
#define FUNC_MODE_SET_BFO_USB	5						//  -   BFO_USB
#define FUNC_MODE_SET_BFO_CWL	6						//  -   BFO_CWL
#define FUNC_MODE_SET_BFO_CWU	7						//  -   BFO_CWU
#define FUNC_MODE_SET_XTALL_1	8						//  -      VFO
#define FUNC_MODE_SET_XTALL_2	9						//  -      BFO
#define FUNC_MODE_SET_DRV_ST_1	10						//  -     CLK0 Si5351  VFO
#define FUNC_MODE_SET_DRV_ST_2	11						//  -     CLK0 Si5351  BFO
#define FUNC_MODE_SET_CONV		12						//  -   
#define FUNC_MODE_SET_SWP_F_TX	13						//  -  /   VFO/BFO_xxx  TX
#define FUNC_MODE_SET_BAND		14						//  -  / 

#define BAND_160m				1						// 160m
#define BAND_80m				2						// 80m
#define BAND_40m				3						// 40m
#define BAND_30m				4						// 30m
#define BAND_20m				5						// 20m
#define BAND_17m				6						// 17m
#define BAND_15m				7						// 15m
#define BAND_12m				8						// 15m
#define BAND_10m				9						// 10m

#define BAND_1_MAX				2499999					// 
#define BAND_2_MAX				4499999					//-
#define BAND_3_MAX				7999999					//-
#define BAND_4_MAX				11999999				//-
#define BAND_5_MAX				15999999				//-
#define BAND_6_MAX				19999999				//-
#define BAND_7_MAX				23999999				//-
#define BAND_8_MAX				25999999				//-

#define BUT_NULL				0						//
#define BUT_FUNC				1						//-
#define BUT_ATT_PRE				2						//-
#define BUT_PTT					3						//-
#define BUT_MODE				4						//-	

void PTT_ON(int Value);
void ATT_PRE(void);
void Check_Conversion(void);
void Set_MODE(void);
void Band_Label_to_LCD(void);
void Freq_to_LCD(int d);
void Freq_format_to_LCD(uint32_t freq, int col, int str);
void Check_BAND(void);
void Set_BAND(void);

#endif /* MAIN_H_ */