﻿/*
 * HF synthesizer
 * base on Si5351x2
 * 9 - bands
 *
 * Usart.c
 *
 * Author: R9OFG (ex R0AEK)
 * Version - 2.0 beta
 *
 * radio0aek@gmail.com
 * https://r0aek.me
 */ 

#include "Usart.h"

/*объявление констант и переменных---------------------------------------*/

static uint8_t RX_array_Data[32];			//массив принятых байтов для обмена по UART
static uint8_t TX_array_Data[32];			//массив отправляемых байтов для обмена по UART

char strFreqVFO[10];						//для обработки полученной из Rig строки частоты

extern uint8_t func_Mode;					//указатель на переменную
extern unsigned char band_Num;				//указатель на переменную
extern uint32_t freq_VFO;					//указатель на переменную
extern uint8_t ptt_cat_ON_F;				//указатель на переменную
extern uint8_t band_use[9];					//указатель на переменную
extern uint8_t but_ATT_PRE;					//указатель на переменную
extern uint8_t but_MODE;					//указатель на переменную
extern uint16_t timerCountSaveBandData;		//указатель на переменную

/*END_объявление переменных----------------------------------------------*/

void USART_Init(unsigned int ubrr)			//инициализация USART
{
	/*установка скорости передачи*/
	UBRR0H = (unsigned char)(ubrr>>8);
	UBRR0L = (unsigned char)ubrr;
	/*запрещаем прерывания от RX/TX, включение приемника и передатчика*/
	UCSR0B = (0<<RXCIE0)|(0<<TXCIE0)|(1<<RXEN0)|(1<<TXEN0);
	/*настройка и установка формата кадра:asynchronous, no parity, 1-stop bit, 8-bit data*/
	UCSR0C = (0<<UMSEL01)|(0<<UMSEL00)|(0<<UPM01)|(0<<UPM00)|(0<<USBS0)|(0<<UCSZ02)|(1<<UCSZ01)|(1<<UCSZ00);
	_delay_ms(50);
}

void handlerRXUSART(void)					//чтение кольцевого буфера принятых данных из USART
{
	int i = 0;
	uint8_t values;
	//проверяем буфер приема
 	if(rxIn != rxOut)
 	{	
		while((rxIn != rxOut))
		{
			//читаем буфер приема пока индексы не сравняются
			values = (bufRX[rxOut++]);
			rxOut &= BUFMASKRX;
			_delay_ms(1); //без задержки много пропускает!
			
			RX_array_Data[i++] = values;
			if (values == ';')
			{
				i = 0;
				RX_Data_Parsing();
			}
		}
	}
}

void handlerTXUSART(void)					//чтение кольцевого буфера для передачи данных в USART
{
	//проверяем буфер передачи
	if((txIn != txOut) && (UCSR0A & (1 << UDRE0)))
	{
		//в буфере что-то есть, отправляем в UART
		UDR0 = bufTX[txOut++];
		txOut &= BUFMASKTX;
	}
}

void SendByteToCircularBuffer(int idx)		//отправка побайтно в кольцевой буфер для передачи данных в USART
{
	for (int i=0;i<=idx;i++)
	{
		bufTX[txIn++] = TX_array_Data[i];
		txIn &= BUFMASKTX;
	}
}

void CAT_IF_Send(void)						//отправка IF в USART для CAT
{
	ultoa(freq_VFO,strFreqVFO,10);
	TX_array_Data[0] = 'I';
	TX_array_Data[1] = 'F';
	//собираем в строку текущее значение частоты VFO-A
	if (freq_VFO <= 9999999)
	{
		//если частота меньше 10МГц добавляем впереди 4 нуля
		for (int i=2;i<6;i++)
		TX_array_Data[i] = '0';
		//заполняем текущее значение частоты VFO-A
		for (int i=0;i<7;i++)
		TX_array_Data[i+6] = strFreqVFO[i];
	}
	else
	{
		//если частота меньше 10МГц добавляем впереди 3 нуля
		for (int i=2;i<5;i++)
		TX_array_Data[i] = '0';
		//заполняем текущее значение частоты VFO-A
		for (int i=0;i<8;i++)
		TX_array_Data[i+5] = strFreqVFO[i];
	}
	//добавляем пробел
	TX_array_Data[13] = ' ';
	//добавляем RX - 0/TX - 1
	if (func_Mode == FUNC_MODE_NORMAL)
		TX_array_Data[14] = '0';
	else if (func_Mode == FUNC_MODE_TX)
		TX_array_Data[14] = '1';
	//добавляем пробел
	TX_array_Data[15] = ' ';
	//добавляем моду
	switch(but_MODE)
	{
		case 0:
			TX_array_Data[16] = '1';
			break;
		case 1:
			TX_array_Data[16] = '2';
			break;
		case 2:
			TX_array_Data[16] = '7';
			break;
		case 3:
			TX_array_Data[16] = '3';
			break;
	}
	//добавляем в конец терминатор
	TX_array_Data[17] = ';';
	//отправляем строку ответа в кольцевой буфер для отправки в USART
	SendByteToCircularBuffer(17); //17 - количество отправляемых данных из массива в байтах
}

void CAT_ATT_PRE_Send(int Value)			//отправка ATT_PRE в USART для CAT
{
	//отправляем обратно ответ на запрос
	TX_array_Data[0] = 'A';
	if (Value == 1)
	{
		TX_array_Data[1] = 'P';
		TX_array_Data[2] = 'S';
	}
	else
	{
		TX_array_Data[1] = '_';
		TX_array_Data[2] = 'P';	
	}
	switch (but_ATT_PRE)
	{
		case 0:
		TX_array_Data[3] = '0';
		break;
		case 1:
		TX_array_Data[3] = '1';
		break;
		case 2:
		TX_array_Data[3] = '2';
		break;
	}
	//добавляем в конец терминатор
	TX_array_Data[4] = ';';
	//отправляем строку ответа в кольцевой буфер для отправки в USART
	SendByteToCircularBuffer(4); //4 - количество отправляемых данных из массива в байтах
}

void RX_Data_Parsing(void)					//обработка данных полученных из USART
{
 	//поступил запрос "IF;" на чтение статуса синтезатора
	{
		if ((RX_array_Data[0] == 'I')&&(RX_array_Data[1] == 'F')&&(RX_array_Data[2] == ';'))
		{
			CAT_IF_Send();
		}
 	}	
	
	//поступил запрос "FA;" на чтение данных о частоте
	{
		if ((RX_array_Data[0] == 'F')&&(RX_array_Data[1] == 'A')&&(RX_array_Data[2] == ';'))
		{
			ultoa(freq_VFO,strFreqVFO,10);
			TX_array_Data[0] = 'F';
			TX_array_Data[1] = 'A';
			if (freq_VFO <= 9999999)
			{
				//если частота меньше 10МГц добавляем впереди 4 нуля
				for (int i=2;i<6;i++)
					TX_array_Data[i] = '0';
				//заполняем значение частоты VFO-A
				for (int i=0;i<7;i++)
					TX_array_Data[i+6] = strFreqVFO[i];
			}
			else
			{
				//если частота меньше 10МГц добавляем впереди 3 нуля
				for (int i=2;i<5;i++)
					TX_array_Data[i] = '0';
				//заполняем значение частоты VFO-A
				for (int i=0;i<8;i++)
					TX_array_Data[i+5] = strFreqVFO[i];
			}		
			//добавляем в конец терминатор
			TX_array_Data[13] = ';';
			//отправляем строку ответа в кольцевой буфер для отправки в USART
			SendByteToCircularBuffer(13); //13 - количество отправляемых данных из массива в байтах
		}
 	}	
	
	//поступил запрос "FAxxxxxxxxxxx;" на изменение частоты VFO-A
	{
		if ((RX_array_Data[0] == 'F')&&(RX_array_Data[1] == 'A')&&(RX_array_Data[13] == ';')/*&&(func_Mode == FUNC_MODE_NORMAL)*/)
		{
			//извлекаем символьное значение заданной частоты в строку
			char freqStr[11];
			for (int i=2;i<13;i++)
				freqStr[i-2] = RX_array_Data[i];
				
			//запоминаем текущее значение VFO
			uint32_t oldFreqVFO = freq_VFO;
			
			//преобразуем полученную строку частоты в число
			freq_VFO = atol(freqStr);
			
			//для WSJT-X/JTDX
			if (freq_VFO == 55) freq_VFO += oldFreqVFO;
			
			
			if ((freq_VFO >= FREQ_VFO_MIN)&&(freq_VFO <= FREQ_VFO_MAX))
			{
				Check_BAND();
				
				//отправляем на LCD
				Freq_format_to_LCD(freq_VFO,6,0);
				
				timerCountSaveBandData = 0;
			}
			else
				freq_VFO = oldFreqVFO;
		}
	}
	
	//поступил запрос "MD2;" - USB или "MD1;" - LSB установка моды SSB
	{
		if ((RX_array_Data[0] == 'M')&&(RX_array_Data[1] == 'D')&&((RX_array_Data[2] == '1')||(RX_array_Data[2] == '2'))&&(RX_array_Data[3] == ';'))
		{
			if (RX_array_Data[2] == '1') but_MODE = 0;
			else if (RX_array_Data[2] == '2') but_MODE = 1;
			//уходим на проверку преобразования
			Check_Conversion();
		}
	}
	
	//поступил запрос "MD7;" - CWL или "MD3;" - CWU установка моды CW
	{
		if ((RX_array_Data[0] == 'M')&&(RX_array_Data[1] == 'D')&&((RX_array_Data[2] == '7')||(RX_array_Data[2] == '3'))&&(RX_array_Data[3] == ';'))
		{
			if (RX_array_Data[2] == '7') but_MODE = 2;
			else if (RX_array_Data[2] == '3') but_MODE = 3;
			//уходим на проверку преобразования
			Check_Conversion();
		}
	}
	
	//поступил запрос "RX;" переход синтезатора в режим RX - PTT off
	{
		if ((RX_array_Data[0] == 'R')&&(RX_array_Data[1] == 'X')&&(RX_array_Data[2] == ';'))
		{
			//PTT off
			if (func_Mode == FUNC_MODE_TX)
			{
				ptt_cat_ON_F = 0;
				PTT_ON(0);
			}
			
			//отправляем ответ обратно, необходимо для WSJTX/JTDX
			TX_array_Data[0] = 'R';
			TX_array_Data[1] = 'X';
			//добавляем в конец терминатор
			TX_array_Data[2] = ';';
			//отправляем строку ответа в кольцевой буфер для отправки в USART
			SendByteToCircularBuffer(2); //2 - количество отправляемых данных из массива в байтах
		}
	}
	
	//поступил запрос "TX;" переход синтезатора в режим TX - PTT on
	{
		if ((RX_array_Data[0] == 'T')&&(RX_array_Data[1] == 'X')&&(RX_array_Data[2] == ';')&&(func_Mode == FUNC_MODE_NORMAL))
		{
			//PTT on
			if (func_Mode == FUNC_MODE_NORMAL)
			{
				ptt_cat_ON_F = 1;
				PTT_ON(1);
			}
		}
	}
		
	/*!!!!!!! ниже идут команды, которых нет в составе OmniRig*/
	
	//поступил запрос "BN0;" смена диапазона вниз
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == 'N')&&(RX_array_Data[2] == '0')&&(RX_array_Data[3] == ';')&&(func_Mode == FUNC_MODE_NORMAL))
		{
			band_Num--;
			if (band_Num < 1) band_Num = 9;
			else
				if (band_Num > 9) band_Num = 1;
			//проверяем, включен ли диапазон в настройках, если нет, то пропускаем
			while (band_use[band_Num - 1] != 1)
			{
				band_Num--;
				if (band_Num < 1) band_Num = 9;
				else
					if (band_Num > 9) band_Num = 1;
			}
			//сохраняем номер текущего диапазона в EEPROM
			eeprom_write_byte((uint8_t*)0x03,band_Num);
			//переходим на текущий диапазон
			Set_BAND();
			
			//отправляем обратно ответ на запрос
			/*TX_array_Data[0] = 'B';
			TX_array_Data[1] = 'N';
			TX_array_Data[2] = '0';
			//добавляем в конец терминатор
			TX_array_Data[3] = ';';
			//отправляем строку ответа в кольцевой буфер для отправки в USART
			SendByteToCircularBuffer(3); //3 - количество отправляемых данных из массива в байтах*/
		}
	}
	
	//поступил запрос "BN1;" смена диапазона вверх
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == 'N')&&(RX_array_Data[2] == '1')&&(RX_array_Data[3] == ';')&&(func_Mode == FUNC_MODE_NORMAL))
		{
			band_Num++;
			if (band_Num > 9) band_Num = 1;
			else
				if (band_Num < 1) band_Num = 9;
			//проверяем, включен ли диапазон в настройках, если нет, то пропускаем
			while (band_use[band_Num - 1] != 1)
			{
				band_Num++;
				if (band_Num > 9) band_Num = 1;
				else
					if (band_Num < 1) band_Num = 9;
			}
			//сохраняем номер текущего диапазона в EEPROM
			eeprom_write_byte((uint8_t*)0x03,band_Num);
			//переходим на текущий диапазон
			Set_BAND();
			
			//отправляем обратно ответ на запрос
			/*TX_array_Data[0] = 'B';
			TX_array_Data[1] = 'N';
			TX_array_Data[2] = '1';
			//добавляем в конец терминатор
			TX_array_Data[3] = ';';
			//отправляем строку ответа в кольцевой буфер для отправки в USART
			SendByteToCircularBuffer(3); //3 - количество отправляемых данных из массива в байтах*/
		}
	}
	
	//поступил запрос "A_P;" смена режимов ATT/PRE
	{
		if ((RX_array_Data[0] == 'A')&&(RX_array_Data[1] == '_')&&(RX_array_Data[2] == 'P')&&(RX_array_Data[3] == ';')&&(func_Mode == FUNC_MODE_NORMAL))
		{
			but_ATT_PRE++;
			if (but_ATT_PRE > 2)
				but_ATT_PRE = 0;
			ATT_PRE();	//отправка обратно ответа на запрос - в функции ATT_PRE()			
		}
	}
	
	//поступил запрос "APS;" статус режимов ATT/PRE
	{
		if ((RX_array_Data[0] == 'A')&&(RX_array_Data[1] == 'P')&&(RX_array_Data[2] == 'S')&&(RX_array_Data[3] == ';')&&(func_Mode == FUNC_MODE_NORMAL))
		{
			CAT_ATT_PRE_Send(1);			
		}
	}
	
	//очищаем массив от старых значений
	for(int i=0;i<16;i++)
		RX_array_Data[i] = ' ';
}